#ifndef _MATH_3D_H
#define _MATH_3D_H

#include <math.h>

class Matrix3x3;
class Matrix4x4;
class Quaternion;

class Vector3d
{
public:
    float x,y,z; 
    
    Vector3d(){}
    Vector3d(float X,float Y,float Z){x=X;y=Y;z=Z;}
    
    void operator+=(Vector3d &vec);
    void operator-=(Vector3d &vec);
    void operator*=(float scalar);
    void operator/=(float scalar);
    
    float Length();
    void Normalize();
    void Invert();
    
    float Dot(Vector3d &vec);     //returns dot product between this & argument vector
    Vector3d Cross(Vector3d &vec);//returns cross product between this & argument vector
    
    void MultiplyBy(Matrix3x3 &mat);//multiplies vector by 3x3 matrix
    void MultiplyBy(Matrix4x4 &mat);//multiplies vector by 4x4 matrix
           
};

Vector3d operator+(Vector3d A,Vector3d B);
Vector3d operator-(Vector3d A,Vector3d B);
Vector3d operator*(Vector3d A,float B);
Vector3d operator*(float B,Vector3d A);
Vector3d operator/(Vector3d A,float B);

class Matrix3x3
{
public:
    float M[9];
    
    void LoadIdentity();//sets this matrix to 3x3 identity
    void Transpose();   //transposes the matrix
    void Invert();      //inverts the matrix
    void MultiplyBy(Matrix3x3 &mat); //multiplies this matrix by a 3x3 matrix
    
    void Convert(Quaternion &quat);  //creates this matrix from argument quaternion
};


class Matrix4x4
{
public:
    float M[16];
    
    void LoadIdentity();//sets this matrix to 4x4 identity
    void Transpose();   //transposes the matrix
    void Invert();      //inverts the matrix
    void MultiplyBy(Matrix4x4 &mat); //multiplies this matrix by a 4x4 matrix
};

class Quaternion
{
public:
    float x,y,z,w;
    
    float Magnitude();   //returns magrintude of quaternion
    void Normalize();    //normalizes the quaternion
    void Invert();       //inverts the quaternion
    void MultiplyBy(Quaternion &quat);    //multiplies this quaternion by another quaternion
    void Convert(Matrix3x3 &mat);  //creates this quaternion from a 3x3 matrix
    
    void Slerp(Quaternion &quat,float t); //slerps this quaternion toward quat
    void Slerp(Quaternion &quat1,Quaternion &quat2,float t); //sets this quaternion
                                             //to the result of slerping from quat1 to quat2
};    

#endif

